﻿using Com.Bigdata.Dis.Sdk.DISCommon.Model;
using DISClient_3._5.Client;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;

namespace ConsoleApplication_3_5
{
    class Program
    {
        public static void Main(string[] args)
        {
            string streamName = "streamName";
            const string shardId = "shardId-0000000001";
            string appName = "testAppName";

            //创建通道
            try
            {
                DISClient.CreateStream(streamName, 1);
                // DISClient.CreateStreamWithOBS("dis-obs", 2);
                // DISClient.CreateCustomObsStream("dis-obsfile", 2);
                // DISClient.CreateStreamWithMRS("dis-mrs", 1);
                // DISClient.CreateStreamWithDLI("dis-dli", 1);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //上传数据
            try
            {
                DISClient.RunProducerDemo(streamName, shardId);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }


            //查询通道列表
            try
            {
                DISClient.DescribeStreamList(null, 100);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //查询通道详情
            try
            {
                DISClient.DescribeStream(streamName, null, 100);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //下载数据
            try
            {
                DISClient.RunConsumerDemo(streamName, shardId);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //获取数据游标
            try
            {
                DISClient.GetCursorDemo(streamName, shardId);
            }
            catch (Exception e)
            {
                Console.Out.WriteLine(e.Message);
            }

            //创建APP
            try
            {
                ResponseResult response = DISClient.CreateAPPDemo(appName);
                Console.Out.WriteLine(response);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //新增Checkpoint
            try
            {
                DISClient.AddCheckPointDemo(streamName, appName);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //查询Checkpoint
            try
            {
                DISClient.GetCheckPointDemo(streamName, appName);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //变更分区数量
            //Custom file stream is not suitable for operating.
            try
            {
                DISClient.UpdatePartitionCount(streamName, 3);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            //删除通道
            try
            {
                DISClient.DeleteStream(streamName);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }





        }
    }

    public class DISClient
    {
        #region DescribeStream
        /// <summary>
        /// 查询通道列表
        /// </summary>
        /// <param name="startStreamName">起始通道名称</param>
        /// <param name="limit">每次查询时返回的通道数量</param>
        /// <returns></returns>
        public static DescribeStreamListResult DescribeStreamList(string startStreamName, int? limit)
        {
            DescribeStreamListResult response = null;
            var dic = new DISIngestionClient();
            var request = new DescribeStreamRequest();
            if (!string.IsNullOrEmpty(startStreamName))
            {
                request.StartStreamName = startStreamName;
            }

            if (limit != null)
            {
                request.Limit = limit.Value;
            }

            response = dic.DescribeStreamList(request);
            var reqJson = JsonConvert.SerializeObject(response);
            Console.WriteLine(reqJson);
            return response;
        }

        /// <summary>
        /// 查询通道详情
        /// </summary>
        /// <param name="streamName">通道名称 </param>
        /// <param name="startPartitionId">起始分区ID </param>
        /// <param name="limitPartitions">单次请求返回的最大分区数 </param>
        /// <returns></returns>
        public static DescribeStreamResult DescribeStream(string streamName, string startPartitionId, int? limitPartitions)
        {
            DescribeStreamResult response = null;
            var dic = new DISIngestionClient();
            var request = new DescribeStreamRequest { StreamName = streamName };

            if (!string.IsNullOrEmpty(startPartitionId))
            {
                request.StartPartitionId = startPartitionId;
            }

            if (limitPartitions != null)
            {
                request.LimitPartitions = limitPartitions.Value;
            }

            response = dic.DescribeStream(request);
            var responseJson = JsonConvert.SerializeObject(response);
            Console.WriteLine(responseJson);
            return response;
        }
        #endregion

        #region DeleteStream
        /// <summary>
        /// 删除通道
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <returns></returns>
        public static ResponseResult DeleteStream(string streamName)
        {
            var dic = new DISIngestionClient();
            var request = new DescribeStreamRequest { StreamName = streamName };
            ResponseResult response = dic.DeleteStream(request);
            Console.WriteLine(response);
            return response;
        }
        #endregion

        #region CreateStream
        /// <summary>
        /// 创建无转储任务的通道
        /// </summary>
        /// <param name="streamName"> 通道名称</param>
        /// <param name="partitionCount">分区数</param>
        /// <returns></returns>
        public static ResponseResult CreateStream(string streamName, int partitionCount)
        {
            DISIngestionClient dic = new DISIngestionClient();
            var request = new CreateStreamRequest
            {
                //通道名称
                StreamName = streamName,
                //通道类型
                StreamType = "ADVANCED",
                //分区数量
                PartitionCount = partitionCount,
                //源数据类型
                DataType = "JSON",//"BLOB",
                //生命周期
                DataDuration = 7 * 24,
            };

            //源数据转储为parquet和carbon格式时必选，如果不设置，则注释该代码片段
            if (request.DataType.Equals("JSON") || request.DataType.Equals("CSV"))
            {
                request.DataSchema = "{\"type\":\"record\",\"name\":\"RecordName\",\"fields\":[{\"name\":\"key\",\"type\":\"string\",\"doc\":\"Type inferred from '\\\"value\\\"'\"}]}";
            }

            ResponseResult response = dic.CreateStream(request);
            Console.WriteLine(response);
            return response;
        }

        /// <summary>
        /// 创建具有OBS按周期转储任务的通道
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="partitionCount">分区数</param>
        /// <returns></returns>
        public static ResponseResult CreateStreamWithOBS(string streamName, int partitionCount)
        {
            var dic = new DISIngestionClient();
            var request = new CreateStreamRequest
            {
                StreamName = streamName,
                StreamType = "COMMON",
                PartitionCount = partitionCount,
                DataType = "BLOB",
                DataDuration = 24,
            };

            List<ObsDestinationDescriptorEntity> obsDestinationDescriptorEntitys = new List<ObsDestinationDescriptorEntity>();

            var obsDestinationDescriptorEntity = new ObsDestinationDescriptorEntity
            {
                //IAM委托名称
                AgencyName = "shawntestiam",
                //桶名称
                ObsBucketPath = "obs-shawnobstest",
                //自定义文件
                FilePrefix = "thisisshawnfolder",
                //数据导入OBS时间
                DeliverTimeInterval = 30,
                //目录层次结构
                PartitionFormat = "yyyy",
                //分隔符
                RecordDelimiter = "\n"
            };
            obsDestinationDescriptorEntitys.Add(obsDestinationDescriptorEntity);

            request.ObsDestinationDescriptor = obsDestinationDescriptorEntitys;

            ResponseResult response = dic.CreateStream(request);
            Console.WriteLine(response);
            return response;
        }

        /// <summary>
        /// 创建具有OBS自定义文件转储任务的通道
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="partitionCount">分区数</param>
        /// <returns></returns>
        public static ResponseResult CreateCustomObsStream(string streamName, int partitionCount)
        {
            var dic = new DISIngestionClient();
            var request = new CreateStreamRequest
            {
                StreamName = streamName,
                PartitionCount = partitionCount,
                DataDuration = 24,
            };

            List<ObsDestinationDescriptorEntity> obsDestinationDescriptorEntitys = new List<ObsDestinationDescriptorEntity>();
            var obsDestinationDescriptorEntity = new ObsDestinationDescriptorEntity
            {
                AgencyName = "all",
                ObsBucketPath = "obs-shawnobstest",
                DeliverDataType = "file_stream",
            };
            obsDestinationDescriptorEntitys.Add(obsDestinationDescriptorEntity);

            request.ObsDestinationDescriptor = obsDestinationDescriptorEntitys;

            ResponseResult response = dic.CreateStream(request);
            Console.WriteLine(response);
            return response;
        }

        /// <summary>
        /// 创建具有MRS转储任务的通道
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="partitionCount">分区数</param>
        /// <returns></returns>
        public static ResponseResult CreateStreamWithMRS(string streamName, int partitionCount)
        {
            var dic = new DISIngestionClient();
            var request = new CreateStreamRequest
            {
                StreamName = streamName,
                PartitionCount = partitionCount,
                DataDuration = 24,
            };

            List<MRSDestinationDescriptorEntity> mrsDestinationDescriptorEntitys = new List<MRSDestinationDescriptorEntity>();
            var mrsDestinationDescriptorEntity = new MRSDestinationDescriptorEntity
            {
                AgencyName = "shawntestiam",
                MRSClusterId = "0e00362e-cfae-48a3-9344-65e4bb46397d",
                MRSClusterName = "15_verison_test",
                MRSHdfsPath = "/app-logs",
                OBSBucketPath = "obs-shawn",
                FilePrefix = "shawnFolder",
                DeliverTimeInterval = 300,
                RetryDuration = 300,
            };
            mrsDestinationDescriptorEntitys.Add(mrsDestinationDescriptorEntity);
            request.MRSDestinationDescriptor = mrsDestinationDescriptorEntitys;

            ResponseResult response = dic.CreateStream(request);
            Console.WriteLine(response);
            return response;
        }

        /// <summary>
        /// 创建具有DLI转储任务的通道
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="partitionCount">分区数</param>
        /// <returns></returns>
        public static ResponseResult CreateStreamWithDLI(string streamName, int partitionCount)
        {
            var dic = new DISIngestionClient();
            var request = new CreateStreamRequest
            {
                StreamName = streamName,
                PartitionCount = partitionCount,
                DataDuration = 24,
            };

            List<DLIDestinationDescriptorEntity> dliDestinationDescriptors = new List<DLIDestinationDescriptorEntity>();
            var dliDestinationDescriptor = new DLIDestinationDescriptorEntity
            {
                AgencyName = "dis_global",
                DLIDatabaseName = "123asd1",
                DLITableName = "dd",
                OBSBucketPath = "obs-shawn",
                DeliverTimeInterval = 300,
                RetryDuration = 300,
            };
            dliDestinationDescriptors.Add(dliDestinationDescriptor);
            request.DLIDestinationDescriptor = dliDestinationDescriptors;

            ResponseResult response = dic.CreateStream(request);
            Console.WriteLine(response);
            return response;
        }
        #endregion

        #region RunProducerDemo
        /// <summary>
        /// 上传数据
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="shardId">分区ID</param>
        public static PutRecordsResult RunProducerDemo(string streamName, string shardId)
        {
            var dic = new DISIngestionClient();

            PutRecordsRequest putRecordsRequest = new PutRecordsRequest();
            putRecordsRequest.StreamName = streamName;
            var putRecordsRequestEntries = new List<PutRecordsRequestEntry>();
            for (int i = 0; i < 3; i++)
            {
                string a = shardId + i;

                var putRecordsRequestEntry = new PutRecordsRequestEntry
                {
                    Data = Encoding.UTF8.GetBytes(a),
                    ExplicitHashKey = "123",
                    PartitionKey = "1",
                    PartitionId = "8"
                };
                putRecordsRequestEntries.Add(putRecordsRequestEntry);
            }
            putRecordsRequest.Records = putRecordsRequestEntries;
            PutRecordsResult response = dic.PutRecords(putRecordsRequest);

            foreach (var item in response.Records)
            {
                Console.WriteLine(item);
            }
            return response;
        }
        #endregion

        #region RunConsumerDemo
        /// <summary>
        /// 下载数据
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="shardId">分区ID</param>
        public static void RunConsumerDemo(string streamName, string shardId)
        {
            var dic = new DISIngestionClient();

            //该参数与游标类型AT_SEQUENCE_NUMBER、AFTER_SEQUENCE_NUMBER搭配使用
            const string startingSequenceNumber = "6";

            //AT_SEQUENCE_NUMBER：从特定序列号所在的记录开始读取。此类型为默认游标类型。
            //AFTER_SEQUENCE_NUMBER：从特定序列号后的记录开始读取。
            //TRIM_HORIZON：从分区中时间最长的记录开始读取。
            //LATEST：在分区中最新的记录之后开始读取，以确保始终读取分区中的最新数据。
            //AT_TIMESTAMP：从特定时间戳开始读取。
            const string shardIteratorType = "AT_SEQUENCE_NUMBER";

            var request = new GetShardIteratorRequest
            {
                StreamName = streamName,
                ShardId = shardId,
                //可选参数
                StartingSequenceNumber = startingSequenceNumber,
                //可选参数
                ShardIteratorType = shardIteratorType,
            };

            var recordsRequest = new GetRecordsRequest();
            var response = dic.GetShardIterator(request);
            Console.Out.WriteLine(response);

            var iterator = response.ShardIterator;

            while (true)
            {
                recordsRequest.ShardIterator = iterator;
                var recordResponse = dic.GetRecords(recordsRequest);
                if (recordResponse.Records.Count > 0)
                {
                    foreach (var record in recordResponse.Records)
                    {
                        Console.WriteLine("Record[{0}] = {1}", record.SequenceNumber, DecodeData(record.Data));
                    }
                }
                else
                {
                    break;
                }
                iterator = recordResponse.NextShardIterator;
            }
        }

        public static string DecodeData(byte[] byteBufferData)
        {
            return Encoding.UTF8.GetString(byteBufferData);
        }
        #endregion

        #region GetCursorDemo
        /// <summary>
        /// 获取数据游标
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="shardId">分区ID</param>
        public static GetShardIteratorResult GetCursorDemo(string streamName, string shardId)
        {
            var dic = new DISIngestionClient();
            const string startingSequenceNumber = "0";
            const string shardIteratorType = "TRIM_HORIZON";

            var request = new GetShardIteratorRequest
            {
                StreamName = streamName,
                ShardId = shardId,
                ShardIteratorType = shardIteratorType,
                StartingSequenceNumber = startingSequenceNumber,
            };

            var response = dic.GetShardIterator(request);
            Console.Out.WriteLine(response);
            return response;
        }
        #endregion

        #region CreateAPPDemo
        /// <summary>
        /// 创建APP
        /// </summary>
        /// <param name="appName">app名称</param>
        public static ResponseResult CreateAPPDemo(string appName)
        {
            var dic = new DISIngestionClient();
            var request = new AppRequest
            {
                AppName = appName,
            };

            ResponseResult response = dic.CreateApp(request);
            return response;
        }
        #endregion

        #region AddCheckPoint
        /// <summary>
        /// 新增Checkpoint
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="appName">app名称</param>
        public static ResponseResult AddCheckPointDemo(string streamName, string appName)
        {
            var dic = new DISIngestionClient();
            var request = new CommitCheckpointRequest
            {
                stream_name = streamName,
                AppName = appName,
                partition_id = "shardId-0000000000",
                sequence_number = "10",
                metadata = "metadata",
                checkpoint_type = "LAST_READ",
            };

            var response = dic.CommitCheckpoint(request);
            Console.Out.WriteLine(response);
            return response;
        }
        #endregion

        #region GetCheckPoint
        /// <summary>
        /// 查询Checkpoint
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="appName">app名称</param>
        public static GetCheckpointResult GetCheckPointDemo(string streamName, string appName)
        {
            var dic = new DISIngestionClient();
            var request = new GetCheckpointRequest
            {
                StreamName = streamName,
                AppId = appName,
                ShardId = "shardId-0000000000",
                CheckpointType = "LAST_READ",
            };

            var response = dic.GetCheckpoint(request);
            Console.Out.WriteLine(response);
            return response;
        }
        #endregion

        #region UpdatePartitionCount
        /// <summary>
        /// 变更分区数量
        /// </summary>
        /// <param name="streamName">通道名称</param>
        /// <param name="count">变更的目标分区数量</param>
        public static UpdateShardsResult UpdatePartitionCount(string streamName, int count)
        {
            var dic = new DISIngestionClient();
            var request = new UpdateShardsRequest
            {
                StreamName = streamName,
                TargetPartitionCount = count,
            };

            var response = dic.UpdatePartition(request);
            Console.Out.WriteLine(response);
            return response;
        }
        #endregion
    }
}
